#! /usr/bin/env python

"""
Remote RCS -- command line interface
"""

import sys
import os
import getopt
import string
import tempfile
import copy
from workdir import DirSupport
from prcslib import PRCSWorkDir
from prcsclient import openprcsclient

wd = None

def main():
    global wd
    try:
	opts, rest = getopt.getopt(sys.argv[1:], 'h:p:qvL')
	if not rest:
	    cmd = 'info'
	else:
	    cmd, rest = rest[0], rest[1:]
	if not commands.has_key(cmd):
	    raise getopt.error, "unknown command"
	coptset, func = commands[cmd]
	copts, files = getopt.getopt(rest, coptset)
	try:
	    wd = PRCSWorkDir()
	    project = wd.project
	    rev = wd.version
	    # force a new project if it is assigned manually for cmds permitted
	    if cmd in ['co', 'get', 'info', 'log'] and \
	       files and not os.path.exists(files[0]):
		if files[0] == project:
		    files = files[1:]
		else:
		    wd = None
		    project, files = files[0], files[1:]
		    rev = None
	except IOError:
	    if cmd in ['check', 'ci', 'put', 'diff', 'clean']:
		print 'This command is unavailable in an alien directory.'
		return
            elif not cmd in ['list']:
	        if len(files) == 0:
		    print 'You must provide at least the project name.'
		    return
	        project, files = files[0], files[1:]
	        rev = None
            else:
                project = rev = None
    except getopt.error, msg:
	print msg
	print "usage: rprcs [options] command [options] [project] [file1] [file2]..."
	print "where command can be:"
        print "      list        # list all the available projects"
	print "      info        # print project info"
	print "      log         # print log for individual file"
	print "      check       # check status of files"
	print "      co|get      # checkout"
	print "      diff        # diff rcs file and work file"
	print "      ci|put      # checkin the given files"
	print "      clean       # cleanup the working directory"
	# for cmd in commands.keys():
	#     print cmd, commands[cmd][0]
	sys.exit(2)

    x = openprcsclient(opts)
    if wd: wd.setproxy(x)
    try:
	func(x, copts, project, rev, files)
    except (IOError, os.error), msg:
	print msg


def list(x, copts, project, rev, files):
    i = 0
    print '\nAvaiable projects:'
    for p in x.list():
	i = i + 1
	print '%s) \t' % i, p


def info(x, copts, project, rev, files):
    wrev = rev
    if not files: rev = None		# force info provide info for all
					# revs by default when no files present

    rev, flags = processcopts(copts, rev)

    print x.info(project, rev, files, flags)

    print '\nThe working version is: %s(w)' % wrev
    print 'Current branch head is:', x.getbranchhead(project, rev)


def log(x, copts, project, rev, files):
    rev, flags = processcopts(copts, rev)
    print x.log(project, rev, files, '-r:')
    

def check(x, copts, project, rev, files):

    wrev = rev

    optm = optu = optx = 0
    for o, a in copts:
	if o == '-r':
	    rev = a
	elif o == '-a':
	    optm = optu = optx = 1
	elif o == '-x':
	    optx = 1
	elif o == '-m':
	    optm = 1
	elif o == '-u':
	    optu = 1
	    
    if not optx and not optu and not optm:
	if files:
	    optm = optu = optx = 1
	else:
	    optm = 1

    rev = x.normalizerev(project, rev)
    print 'Changes made from %s to %s(w):' % (rev, wrev)
    report = wd.check(rev, files)
    keys = report.keys()
    keys.sort()
    
    for fn in keys:
	if fn == project + '.prj': continue
	if optu and report[fn] == '===':
	    print report[fn], '\t', fn
	elif optm and ( report[fn] == 'Mod' or report[fn] == 'Rmv'
			or report[fn] == 'Add' or report[fn] == 'Omt' ):
	    print report[fn], '\t', fn
	elif optx and ( report[fn] == 'Xtr' or report[fn] == '???' ):
	    print report[fn], '\t', fn


def checkout(x, copts, project, rev, files):
    # checkout is special that it may have wd.project != project
    # or it may have wd = None
    wrev = rev
    optp = 0
    flags = ''
    for o, a in copts:
	if o == '-r':
	    rev = a
	elif o == '-p':
	    optp = 1
	else:
	    flags = flags + o
 
    sys.stderr.write('The working version is: %s\n' % wrev)
    sys.stderr.write('The version to be checked out: %s\n' % x.normalizerev(project, rev))

    rsums = x.getsums(project, rev)
    if not files:
	files = rsums.keys() + [project + '.prj']
	
    if wd:
	report = wd.check(rev, files)

    for fn in files:
	if optp:
	    data = x.get(project, rev, fn, flags)
	    sys.stdout.write(data)
	else:
	    if wd and report[fn] == '===': continue
	    print 'Get ', fn, '...'
	    data = x.get(project, rev, fn)
	    if os.path.exists(fn):
		ok = raw_input("%s will be overwritted. Continue(Y/n)? " % fn)
		if not ok or ok in  'Yy':
		    f = open(fn, 'w')
		else: continue
	    else:
		f = open(fn, 'w')
	    f.write(data)
	    f.close()


def diff(x, copts, project, rev, files):
    rev, flags = processcopts(copts, rev)
	    
    report = wd.check(rev, files)

    for fn in report.keys():
	if report[fn] == 'Mod':
	    data = x.get(project, rev, fn)
	    tfn = tempfile.mktemp()
	    try:
		tf = open(tfn, 'w')
		tf.write(data)
		tf.close()
		print "\n===> %s <===\n" % fn
		os.system('diff %s %s %s' % (flags, fn, tfn))
	    finally:
		remove(tfn)


def checkin(x, copts, project, rev, files = []):
    rev, flags = processcopts(copts, rev)
    p = string.find(rev, '.')
    if p < 0: Mrev = rev
    else: Mrev = rev[:p]

    if not files:
	files = wd.filesinproject()
    report = wd.check(rev, files)
    cfiles = copy.copy(files)
    
    for f in files:
	if report[f] == 'Omt':
	    print 'File %s is missing. Stop.' % f
	    return
	elif report[f] == '???':
	    print 'File %s is unknown. Stop.' % f
	    return
	elif report[f] == '===':
	    cfiles.remove(f)

    if not cfiles:
	print 'No modifications are found. Stop.'
	return

    ok = raw_input('>> Edit the log for project %s (Y/n)? ' % project)
    if not ok or ok in 'Yy':
	os.system('vi %s.prj +/New-Version-Log' % project)
    fn = project + '.prj'
    data = open(fn).read()
    msg = x.put(fn, data)
    if msg: print msg

    log = {}
    for fn in cfiles:
	print 'Checking in', fn, '...'
	if report[fn] != 'Add':
	    log[fn] = asklogmessage(0)
	data = open(fn).read()
	msg = x.put(fn, data)
	if msg: print msg
    
    st = x.checkin(project, rev, cfiles)
    if st:
	print st
    else:
	for fn in log.keys():
	    x.edit(project, rev, fn, log[fn])
	# update the local .prj file at last
	checkout(x, [('-r', Mrev)], project, Mrev, [project+'.prj'])
    

def edit(x, copts, project, rev, files):
    for fn in files:
	log = asklogmessage(0)
	x.edit(project, rev, fn, log)


def clean(x, copts, project, rev, files):
    # Note, .* files will always be kept
    files = DirSupport().recursivelistfiles(igns=['.*', '*.pyc'])
    keepfiles = wd.getdescriptor().keys() + [project + '.prj']
    for fn in files:
	if fn in keepfiles: continue
	ok = raw_input("%s will be removed. Continue(Y/n)? " % fn)
	if not ok or ok in 'Yy':
	    remove(fn)


def asklogmessage(new):
    if new:
	print "enter description,",
    else:
	print "enter log message,",
    print "terminate with single '.' or end of file:"
    if new:
	print "NOTE: This is NOT the log message!"
    message = ""
    while 1:
	sys.stderr.write(">> ")
	sys.stderr.flush()
	line = sys.stdin.readline()
	if not line or line == '.\n': break
	message = message + line
    if blank(message): message = '<none>'
    return message


def blank(str):
    for ch in str:
	if not ch in string.whitespace: return 0
    return 1


def processcopts(copts, rev):
    flags = ''
    for o, a in copts:
	if o == '-r':
	    rev = a
	else:
	    flags = flags + ' ' + o + a
    flags = flags[1:]
    return (rev, flags)

    
def remove(fn):
    try:
	os.unlink(fn)
    except os.error:
	pass


commands = {
    'info': ('r:lL', info),
    'list': ('', list),
    'log': ('r:', log),
    'ci': ('r:lL', checkin),
    'put': ('r:lL', checkin),
    'co': ('r:pt', checkout),
    'get': ('r:pt', checkout),
    'edit': ('', edit),
    'check': ('r:axmu', check),
    'clean': ('', clean),
    'diff': ('r:c', diff),
    }


if __name__ == '__main__':
    main()
