#! /usr/bin/env python

"Remote WorkDir -- command line interface"

import sys
import os
import getopt
import tempfile
import workdir
from wdclient import openwdclient

def main():
	try:
		opts, rest = getopt.getopt(sys.argv[1:], 'h:p:d:qvL')
		if not rest:
			cmd = 'list'
		else:
			cmd, rest = rest[0], rest[1:]
		if not commands.has_key(cmd):
			raise getopt.error, "unknown command"
		coptset, func = commands[cmd]
		copts, files = getopt.getopt(rest, coptset)
	except getopt.error, msg:
		print msg
		print "usage: rwd [options] command [options] [file] ..."
		print "where command can be:"
		print "      cd          # goto directory"
		print "      put         # put the given files"
		print "      get         # get the given files"
		print "      list        # list available directories"
		print "      check       # check"
		print "      diff        # diff rcs file and work file"
		print "if no files are given, list is assumed"
		sys.exit(2)
	rwd = workdir.RWorkDir()
	x = openwdclient(opts)
	rwd.setproxy(x)
	try:
	    func(rwd, copts, files)
	except (IOError, os.error), msg:
	    print msg


def list(rwd, copts, files):
    keys = rwd.proxy.listwd().keys()
    keys.sort()
    print 'Available directories:', keys
    print 'Current directory:', rwd.proxy.pwd(), '\n'

    i = 0
    for f in rwd.proxy.workfiles():
	i = i + 1
	print '%s)\t' % i, f


def cd(rwd, copts, files):
    if files:
	rwd.proxy.chwd(files[0])


def check(rwd, copts, files):
    optm = optu = optn = optr = 0
    for o, a in copts:
        if o == '-a':
            optm = optu = optn = optr = 1
        elif o == '-m':
            optm = 1
        elif o == '-u':
            optu = 1
        elif o == '-n':
            optn = 1
        elif o == '-r':
            optr = 1
    if not optu and not optm and not optn and not optr:
	optm = optn = optr = 1
	    
    report = rwd.check(files)
    keys = report.keys()
    keys.sort()

    for fn in keys:
	if optu and report[fn] == '===':
            print report[fn], '\t', fn
        elif optm and report[fn] == 'Mod':
            print report[fn], '\t', fn
	elif optr and report[fn] == 'Rmv':
	    print report[fn], '\t', fn
	elif optn and report[fn] == 'New':
            print report[fn], '\t', fn

	
def put(rwd, copts, files):
    report = rwd.check(files)
    keys = report.keys()
    keys.sort()

    for fn in keys:
	if report[fn] == '===' or report[fn] == 'Rmv': continue
	print 'Put', fn, '...'
	lmode, latime, lmtime, ldate = rwd.stamp(fn)
	if report[fn] == 'Mod': 
	    rmode, ratime, rmtime, rdate = rwd.proxy.stamp(fn)
	    if lmtime < rmtime:
		print 'Local:\t', ldate
		print 'Remote:\t', rdate
		ok = raw_input('Remote file is newer. Are you sure to overwrite (Y/n)? ')
		if ok and not ok in 'Yy':
		    continue
	    rwd.proxy.backup(fn)
	f = open(fn)
	data = f.read()
	f.close()
	rwd.proxy.put(fn, data)
	rwd.proxy.setstamp(fn, lmode, latime, lmtime)

def get(rwd, copts, files):
    optp = 0
    for o, a in copts:
	if o == '-p':
	    optp = 1

    report = rwd.check(files)
    keys = report.keys()
    keys.sort()

    for fn in keys:
        if optp:
            data = rwd.proxy.get(fn)
            sys.stdout.write(data)
        else:
	    if report[fn] == '===' or report[fn] == 'New': continue
            print 'Get ', fn, '...'
	    rmode, ratime, rmtime, rdate = rwd.proxy.stamp(fn)
	    if report[fn] == 'Mod':
		lmode, latime, lmtime, ldate = rwd.stamp(fn)
		if lmtime > rmtime:
		    print 'Local:\t', ldate
		    print 'Remote:\t', rdate
		    ok = raw_input('Local file is later. Are you sure to continue (Y/n)? ')
		    if ok and not ok in 'Yy':
			continue
            data = rwd.proxy.get(fn)
	    rwd.backup(fn)
	    f = open(fn, 'w')
            f.write(data)
            f.close()
	    rwd.setstamp(fn, rmode, ratime, rmtime)


def diff(rwd, copts, files):
    flags = ''
    for o, a in copts:
	flags = flags + ' ' + o + a
    flags = flags[1:]

    report = rwd.check(files)
    keys = report.keys()
    keys.sort()

    for fn in keys:
	if report[fn] == 'Mod':
	    data = rwd.proxy.get(fn)
	    tfn = tempfile.mktemp()
	    try:
		tf = open(tfn, 'w')
		tf.write(data)
		tf.close()
		print "\n===> %s <===\n" % fn
		os.system('diff %s %s %s' % (flags, tfn, fn))
	    finally:
		rwd.rmfile(tfn)


def pwd(rwd, copts, files):
    print rwd.proxy.pwd()


def clean(rwd, copts, pats):
    rwd.proxy.clean(pats)
    rwd.clean(pats)
    
commands = {
        'cd': ('', cd), 
	'pwd': ('', pwd),
	'list': ('', list),
	'check': ('umnra', check),
	'diff': ('c', diff),
	'clean': ('', clean),
	'put': ('', put),
	'get': ('p', get),
	}


if __name__ == '__main__':
	main()
